<?php

/**
 * Resizes images.
 *
 * This simple script was originally written by Gunnar Lium (post@gunnarlium.com).
 * Updated by yvind Bakkeli (olivaq@gmail.com) and Paal Braathen (paalbra@gmail.com).
 *
 * Usage: <img src="imageResize.php?pic=source.type&amp;maxwidth=xxx>
 * Replace source.type with the relevant imagesource and xxx with desired maxwidth.
 *
 * This script supports .gif, .jpg and .png fmeormats, but further extensions should be
 * quite easy to implement.
 *
 * Please feel free to modify and use this script at your own leisure!
 *
 * Update 18/03-05: Added support for transparent gifs/pngs.
 * Update 04/10-09: Added croppingoption for scaling with both width and height constrictions -yvind Bakkeli
 * Update 23/10-09: Added support for cropping away the sides of a picture as well -Paal Braathen
 *
 * @param pic The picture to be resized
 * @param maxwidth The maximum width of the output
 * @param maxheight The maximum height of the output
 * @param crop If set, output will be cropped to fit maxwidth and maxheight
 * @param offsety Not implemented at the moment
 * @param offsetx Not implemented at the moment
 ***********************************************************************************/


// set directory for cache
$image_cache = 'image_cache/';

if(!file_exists($_GET['pic']))
{
	//if image dosn't exist, use default!
	$tmp = glob($_GET['pic'] . '*');
	if($tmp) {
		$_GET['pic'] = array_pop($tmp);
    } else {
		$_GET['pic'] = 'images/news/default.jpg';
    }
}

// get modification time for original (used in hash) - in case the image if modified
// we want to make a cache of the new image.
$filemtime = file_exists($_GET['pic'])?filemtime($_GET['pic']):0;

// make a hash of query request and modification time that is used for image cache name
$hash = md5($_SERVER['QUERY_STRING'] . $filemtime);

// concat imagecache path and hash
$cachename = $image_cache . $hash;


//check if cachefile exist, file extension is added afterwards so there's a wildcard there
if($src = glob($cachename . '.*'))
{
	$tmp =
	//fetch a/the element from the glob
	$img = array_pop($src);
	//get format for image
	$format = array_pop(explode('.', $img));
	
	header('Content-type: image/' . $format);
	
	//output image
        echo file_get_contents($img);
	
	// flush before checking for older images
	// in case things take time
        ob_flush();
        flush();

	//try to check if we need to check for unused images in the cache
        if(file_exists($image_cache . 'lastcheck') && fileatime($image_cache . 'lastcheck') + 3600 < time())
        {       
		touch($image_cache . 'lastcheck');
		/*
		function logcache($str)
		{
			@file_put_contents('imagecache.log', date("c\t") . $str . "\n", FILE_APPEND);
		}		
		*/
		$removed = 0;
                $files = glob($image_cache . '*.*');
                foreach($files as $file)
                        if(fileatime($file) + 86400 < time()) 
			{
                                unlink($file);
				$removed++;
			}
        	//logcache("Cache cleaned - $removed files removed");	
        }       

        die;
}

function getcachesize()
{
	$tmp = glob($image_cache); // get a list of files in image cache
	if(!is_array($tmp))
		return 0;
	$tmp = array_map(filesize, $tmp); // get sizes of files in list
	return array_sum($tmp); // summarize the sizes and return them
}

if(getcachesize() > 30000000) //if cache is greater than 30 mb panic! 
	die('too large cache');



// Filename and new maxwidth from GET

$filename  = $_GET['pic'];
list($width, $height, $format) = getimagesize($filename);


if (isset($_GET['maxwidth']))
   $maxwidth  = $_GET['maxwidth'];
else $maxwidth = $width;
if (isset($_GET['maxheight']))
   $maxheight = $_GET['maxheight'];
else $maxheight = $height;

$offsetx = 0;
$offsety = 0;
$ratio = $width / $height;
$newratio = $maxwidth / $maxheight;

if (isset($_GET['crop']) && isset($maxwidth) && isset($maxheight)) {
    // TODO Make sure the picture isn't scaled up
    if ($maxwidth > $width) {
        $div = $width / $maxwidth;
        $maxwidth = $width;
        $maxheight = $maxheight * $div;
    }
    if ($maxheight > $height) {
        $div = $height / $maxheight;
        $maxwidth = $maxwidth * $div;
        $maxheight = $height;
    }
    if ($ratio > $newratio) { // Sides should be cut off
        $newheight = $height;
        $newwidth = $height * $newratio;
        $offsetx = ($width - $newwidth) / 2;
    } else { // Bottom and top should be cut off
        $newheight = $width / $newratio;
        $newwidth = $width;
        $offsety = ($height - $newheight) / 2;
    }
    $newwidth = $maxwidth;
    $newheight = $maxheight;
    $width = $width - ($offsetx*2);
    $height = $height - ($offsety*2);
} else if (isset($maxwidth) && isset($maxheight)){
    // Get new sizes
    if ($maxwidth > $width) {
        $newwidth = $width;
        $newheight = $height;
    } else {
        $div = $maxwidth / $width;
        $newwidth = $maxwidth;
        $newheight = $height * $div;
    }
    if ($maxheight < $newheight) {
        $div = $maxheight / $newheight;
        $newwidth = $newwidth * $div;
        $newheight = $maxheight;
    }
} else if (isset($maxwidth)) {
    // Get new sizes
    if ($maxwidth > $width) {
        $newwidth = $width;
        $newheight = $height;
    } else {
        $div = $maxwidth/$width;
        $newwidth = $maxwidth;
        $newheight = $height * $div;
    }
} else if (isset($maxheight)) {
    // Get new sizes
    if ($maxheight > $height) {
        $newwidth = $width;
        $newheight = $height;
    } else {
        $div = $maxheight/$height;
        $newwidth = $width * $div;
        $newheight = $height * $div;
    }
}

// Load new image
if ($format != 2){
  $newimage = imagecreate($newwidth, $newheight);
  $color = ImageColorAllocate($newimage, 255, 255, 255);
  imagefill($newimage, 0, 0, $color);
}else {
  $newimage = imagecreatetruecolor($newwidth, $newheight);
}

// Define content type and load source
switch ($format) {
case 1:
    header('Content-type: image/gif');
    $source = imagecreatefromgif($filename);
    break;
case 2:
    header('Content-type: image/jpeg');
    $source = imagecreatefromjpeg($filename);
    break;
case 3:
    header('Content-type: image/png');
    $source = imagecreatefrompng($filename);
    break;
}

// Resize image
imagecopyresampled($newimage, $source, 0, 0, $offsetx, $offsety, $newwidth, $newheight, $width, $height);

// Output image
switch ($format) {
case 1:
    imagegif($newimage, $cachename . '.gif');
    echo file_get_contents($cachename . '.gif');
    break;
case 2:
    imagejpeg($newimage, $cachename . '.jpeg');
    echo file_get_contents($cachename . '.jpeg');
    break;
case 3:
    imagepng($newimage, $cachename . '.png');
    echo file_get_contents($cachename . '.png');
    break;
default:
	die;//not an image? why is it here
}

